package org.testinfinispan.util;

import java.io.IOException;
import java.io.Serializable;
import java.util.Map;
import java.util.Properties;

import org.hibernate.Cache;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.metadata.ClassMetadata;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.service.ServiceRegistryBuilder;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;
import org.jnp.server.Main;
import org.jnp.server.SingletonNamingServer;
import org.testinfinispan.Employee;

public class HibernateUtil {
	private static SessionFactory sessionFactory;
	private static ServiceRegistry serviceRegistry;
	private static EmbeddedCacheManager cacheManager;
	
	
	static {
		cacheManager= getCacheManager();
		try {
			TestJndi.bindCacheManagerToJndi(cacheManager);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		newSessionFactory();
	}
	
	private static EmbeddedCacheManager createCacheManagerFromXml() throws IOException {
		return new DefaultCacheManager("infinispan.xml");
	}
	
	private static SessionFactory buildSessionFactory() {
		try {
		// Create the SessionFactory from hibernate.cfg.xml
		Configuration configuration = new Configuration();
		configuration.configure();
		serviceRegistry = new ServiceRegistryBuilder().applySettings(configuration.getProperties()).buildServiceRegistry();
		return configuration.buildSessionFactory(serviceRegistry);

		}
		catch (Throwable ex) {
			// Make sure you log the exception, as it might be swallowed
			System.err.println("Initial SessionFactory creation failed." + ex);
			throw new ExceptionInInitializerError(ex);
		}
	}
	
    public static void newSessionFactory(){
    	sessionFactory=buildSessionFactory();
    }
    
    public static SessionFactory getSessionFactory() {
        return sessionFactory;
    }
    
    public static Session getSession(){
    	return getSessionFactory().openSession();
    }
    
    public static void shutdown() {
    	// Close caches and connection pools
    	getSessionFactory().close();
    }

    public static void clearLevel2Cache() {
        Cache entityCache = getSessionFactory().getCache();
        if (entityCache == null) {
            return;
        }
        try {
            entityCache.evictQueryRegions();
        } catch (NullPointerException ignored) {
        }

        try {
            entityCache.evictEntityRegions();
        } catch (NullPointerException ignored) {
        }

        try {
            entityCache.evictCollectionRegions();
        } catch (NullPointerException ignored) {
        }
    }
    
    public static void remove(Class c, Serializable key){
    	if (getSessionFactory().getCache().containsEntity(c, key))
    		getSessionFactory().getCache().evictEntity(c,key);
    	else
    		System.err.println("Key:" + key + " does not exist");
    }
   
    
	public static void getCacheNames(){
		if(cacheManager==null) 
			System.err.println("Caches not initialized");
		else 
			System.err.println(cacheManager.getCacheNames());
	}
	
	public static void addListener(String cacheName){
		cacheManager.getCache(cacheName).addListener(new LoggingListener());
	}
	
	public static EmbeddedCacheManager getCacheManager(){
		if (cacheManager==null){
			try {
				cacheManager=createCacheManagerFromXml();
//				cacheManager.getCache().addListener(new LoggingListener());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				System.exit(1);
			}
			//cacheManager.startCaches("net.roseindia.Employee");
		}
		return cacheManager;
	}

}