package org.testinfinispan.util;

import org.infinispan.notifications.Listener;
import org.infinispan.notifications.cachelistener.annotation.CacheEntryCreated;
import org.infinispan.notifications.cachelistener.annotation.CacheEntryInvalidated;
import org.infinispan.notifications.cachelistener.annotation.CacheEntryRemoved;
import org.infinispan.notifications.cachelistener.event.CacheEntryCreatedEvent;
import org.infinispan.notifications.cachelistener.event.CacheEntryInvalidatedEvent;
import org.infinispan.notifications.cachelistener.event.CacheEntryRemovedEvent;
import org.infinispan.util.logging.Log;
import org.infinispan.util.logging.LogFactory;

@Listener
public class LoggingListener {

   private Log log = LogFactory.getLog(LoggingListener.class);

   @CacheEntryCreated
   public void observeAdd(CacheEntryCreatedEvent<?, ?> event) {
      if (!event.isPre()) // So that message is only logged after operation succeeded
         System.err.printf("Cache entry with key %s added in cache %s", event.getKey(), event.getCache());
   }
   @CacheEntryInvalidated
   public void observeInvalidation(CacheEntryInvalidatedEvent<?,?> event){
	   System.err.printf("Cache entry with key %s invalidated in cache %s", event.getKey(), event.getCache());
   }
   @CacheEntryRemoved
   public void observeRemove(CacheEntryRemovedEvent<?, ?> event) {
      System.err.printf("Cache entry with key %s removed in cache %s", event.getKey(), event.getCache());
   }

}
